# the reporoducibility check for tf2 on gpu
import tensorflow as tf
import os
import sys
import pandas as pd
import numpy as np
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import LabelEncoder
from tensorflow.keras import Sequential
from tensorflow.keras.layers import Dense, Dropout, MaxPooling2D, Conv2D, Flatten


def experiment_mnist():
    import tensorflow as tf
    print(tf.__version__)
    batch_size = 128
    epochs = 5
    # set seed as we did for the paper
    exp_id = 0
    seed_val = 1

    from numpy.random import seed
    seed(seed_val)
    tf.random.set_seed(seed_val)

    # set extra seeds just to be sure

    # import random
    # random.seed(seed_val)
    # import tensorflow as tf
    # import os
    # os.environ['PYTHONHASHSEED'] = str(seed_val)


    # model info
    input_shape = (28, 28, 1)
    num_category = 10


    ##model building
    model = Sequential()
    # convolutional layer with rectified linear unit activation
    model.add(Conv2D(32, kernel_size=(3, 3), activation='relu', input_shape=input_shape))
    # 32 convolution filters used each of size 3x3
    # choose the best features via pooling
    model.add(MaxPooling2D(pool_size=(2, 2)))
    # # again
    model.add(Conv2D(64, (3, 3), activation='relu'))
    # # 64 convolution filters used each of size 3x3
    # # choose the best features via pooling
    model.add(MaxPooling2D(pool_size=(2, 2)))
    # # randomly turn neurons on and off to improve convergence
    model.add(Dropout(0.25, seed=seed_val))
    # # flatten since too many dimensions, we only want a classification output
    model.add(Flatten())
    # fully connected to get all relevant data
    model.add(Dense(128, activation='relu'))
    # one more dropout for convergence' sake
    model.add(Dropout(0.5, seed=seed_val))
    # output a softmax to squash the matrix into output probabilities
    model.add(Dense(num_category, activation='softmax'))
    # We use adam as our optimizer
    # categorical ce since we have multiple classes (10)
    model.compile(loss=tf.keras.losses.categorical_crossentropy,
                optimizer="adam",
                metrics=['accuracy'])


    # load mnist dataset
    # (X_train, y_train), (X_test, y_test) = mnist.load_data()
    # fir kaggle 
    a = np.load("data/mnist.npz")
#     # for colab
#     a = np.load("mnist.npz")
    X_test = a['x_test']
    y_test = a['y_test']

    X_train = a['x_train']
    y_train = a['y_train']

    X_train = X_train.reshape(X_train.shape[0], 28, 28, 1)
    X_test = X_test.reshape(X_test.shape[0], 28, 28, 1)
    input_shape = (28, 28, 1)

    # more reshaping
    X_train = X_train.astype('float32')
    X_test = X_test.astype('float32')
    X_train /= 255
    X_test /= 255
    print('X_train shape:', X_train.shape)  # X_train shape: (60000, 28, 28, 1)
    print('X_test shape:', X_test.shape)  # X_train shape: (10000, 28, 28, 1)

    # set number of categories
    num_category = 10
    # convert class vectors to binary class matrices
    y_train = tf.keras.utils.to_categorical(y_train, num_category)
    y_test = tf.keras.utils.to_categorical(y_test, num_category)

    # model training
    model.fit(X_train, y_train, batch_size=batch_size, epochs=epochs,
            verbose=1, validation_data=(X_test, y_test))

    # Print scores
    score = model.evaluate(X_test, y_test, verbose=0)
    print('Test loss:', score[0])
    print('Test accuracy:', score[1])

    f = open('stats.txt', 'w')
    f.write("epochs: %d \n" % (epochs,))
    f.write("val_accuracy: %f \n" % (score[1],))
    f.write("val_loss: %f" % (score[0],))
    f.close()

    # Save the classifications to a .csv file called results.csv
    Y_predicted = model.predict(X_test)
    pred_label = Y_predicted.argmax(axis=1)
    image_id = range(1, len(Y_predicted) + 1)
    df = {'ImageId': image_id, 'Label': pred_label}
    df = pd.DataFrame(df)
    df.to_csv('results.csv', index=False)
    print('GPU is ')
    print(tf.test.is_gpu_available())



if __name__ == '__main__':
    experiment_mnist()